/**
 * Weather Source History API
 * Daily and hourly historical weather data for latitude/longitude points, Zip/Postal Codes, Designated Market Areas, and OnPoint™ points. The Weather Source History API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */

package io.swagger.client.api

import java.text.SimpleDateFormat

import java.util.Date
import io.swagger.client.model.date
import io.swagger.client.model.errorObj
import io.swagger.client.model.historyDmaDayObj
import io.swagger.client.model.historyDmaHourObj
import io.swagger.client.{ApiInvoker, ApiException}

import com.sun.jersey.multipart.FormDataMultiPart
import com.sun.jersey.multipart.file.FileDataBodyPart

import javax.ws.rs.core.MediaType

import java.io.File
import java.util.Date
import java.util.TimeZone

import scala.collection.mutable.HashMap

import com.wordnik.swagger.client._
import scala.concurrent.Future
import collection.mutable

import java.net.URI

import com.wordnik.swagger.client.ClientResponseReaders.Json4sFormatsReader._
import com.wordnik.swagger.client.RequestWriters.Json4sFormatsWriter._

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent._
import scala.concurrent.duration._
import scala.util.{Failure, Success, Try}

import org.json4s._

class DmasApi(
  val defBasePath: String = "https://history.weathersourceapis.com/v2",
  defApiInvoker: ApiInvoker = ApiInvoker
) {
  private lazy val dateTimeFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  private val dateFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  implicit val formats = new org.json4s.DefaultFormats {
    override def dateFormatter = dateTimeFormatter
  }
  implicit val stringReader: ClientResponseReader[String] = ClientResponseReaders.StringReader
  implicit val unitReader: ClientResponseReader[Unit] = ClientResponseReaders.UnitReader
  implicit val jvalueReader: ClientResponseReader[JValue] = ClientResponseReaders.JValueReader
  implicit val jsonReader: ClientResponseReader[Nothing] = JsonFormatsReader
  implicit val stringWriter: RequestWriter[String] = RequestWriters.StringWriter
  implicit val jsonWriter: RequestWriter[Nothing] = JsonFormatsWriter

  var basePath: String = defBasePath
  var apiInvoker: ApiInvoker = defApiInvoker

  def addHeader(key: String, value: String): mutable.HashMap[String, String] = {
    apiInvoker.defaultHeaders += key -> value
  }

  val config: SwaggerConfig = SwaggerConfig.forUrl(new URI(defBasePath))
  val client = new RestClient(config)
  val helper = new DmasApiAsyncHelper(client, config)

  /**
   * A day of weather history for a DMA.
   * Returns a single day of historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param date An date formatted as a RFC3339 date value. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return historyDmaDayObj
   */
  def dmasDmaIdDaysDateGet(dmaId: Integer, date: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[historyDmaDayObj] = {
    val await = Try(Await.result(dmasDmaIdDaysDateGetAsync(dmaId, date, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A day of weather history for a DMA. asynchronously
   * Returns a single day of historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param date An date formatted as a RFC3339 date value. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(historyDmaDayObj)
   */
  def dmasDmaIdDaysDateGetAsync(dmaId: Integer, date: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[historyDmaDayObj] = {
      helper.dmasDmaIdDaysDateGet(dmaId, date, fields, unitScale)
  }

  /**
   * A contiguous range of daily weather history for a DMA.
   * Returns a contiguous range of daily historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
   * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return historyDmaDayObj
   */
  def dmasDmaIdDaysDateStartdateEndGet(dmaId: Integer, dateStart: date, dateEnd: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[historyDmaDayObj] = {
    val await = Try(Await.result(dmasDmaIdDaysDateStartdateEndGetAsync(dmaId, dateStart, dateEnd, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A contiguous range of daily weather history for a DMA. asynchronously
   * Returns a contiguous range of daily historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
   * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, snowDepth, snowfall)* * allPres *(derivatives of mslPres, presTend, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(historyDmaDayObj)
   */
  def dmasDmaIdDaysDateStartdateEndGetAsync(dmaId: Integer, dateStart: date, dateEnd: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[historyDmaDayObj] = {
      helper.dmasDmaIdDaysDateStartdateEndGet(dmaId, dateStart, dateEnd, fields, unitScale)
  }

  /**
   * An hour of weather history for a DMA
   * Returns a single hour of historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return historyDmaHourObj
   */
  def dmasDmaIdHoursTimestampGet(dmaId: Integer, timestamp: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[historyDmaHourObj] = {
    val await = Try(Await.result(dmasDmaIdHoursTimestampGetAsync(dmaId, timestamp, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * An hour of weather history for a DMA asynchronously
   * Returns a single hour of historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(historyDmaHourObj)
   */
  def dmasDmaIdHoursTimestampGetAsync(dmaId: Integer, timestamp: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[historyDmaHourObj] = {
      helper.dmasDmaIdHoursTimestampGet(dmaId, timestamp, fields, unitScale)
  }

  /**
   * A contiguous range of hourly weather history for a DMA.
   * Returns a contiguous range of hourly historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return historyDmaHourObj
   */
  def dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId: Integer, timestampStart: Date, timestampEnd: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[historyDmaHourObj] = {
    val await = Try(Await.result(dmasDmaIdHoursTimestampStarttimestampEndGetAsync(dmaId, timestampStart, timestampEnd, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A contiguous range of hourly weather history for a DMA. asynchronously
   * Returns a contiguous range of hourly historical weather data for a Designated Market Area.
   *
   * @param dmaId A [Designated Market Area ID](https://developer.weathersourceapis.com/apis/supported-dmas/). 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall)* * allPres *(mslPres, presTend, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(historyDmaHourObj)
   */
  def dmasDmaIdHoursTimestampStarttimestampEndGetAsync(dmaId: Integer, timestampStart: Date, timestampEnd: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[historyDmaHourObj] = {
      helper.dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId, timestampStart, timestampEnd, fields, unitScale)
  }

}

class DmasApiAsyncHelper(client: TransportClient, config: SwaggerConfig) extends ApiClient(client, config) {

  def dmasDmaIdDaysDateGet(dmaId: Integer,
    date: date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[historyDmaDayObj]): Future[historyDmaDayObj] = {
    // create path and map variables
    val path = (addFmt("/dmas/{dmaId}/days/{date}")
      replaceAll("\\{" + "dmaId" + "\\}", dmaId.toString)
      replaceAll("\\{" + "date" + "\\}", date.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling DmasApi->dmasDmaIdDaysDateGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def dmasDmaIdDaysDateStartdateEndGet(dmaId: Integer,
    dateStart: date,
    dateEnd: date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[historyDmaDayObj]): Future[historyDmaDayObj] = {
    // create path and map variables
    val path = (addFmt("/dmas/{dmaId}/days/{dateStart},{dateEnd}")
      replaceAll("\\{" + "dmaId" + "\\}", dmaId.toString)
      replaceAll("\\{" + "dateStart" + "\\}", dateStart.toString)
      replaceAll("\\{" + "dateEnd" + "\\}", dateEnd.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling DmasApi->dmasDmaIdDaysDateStartdateEndGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def dmasDmaIdHoursTimestampGet(dmaId: Integer,
    timestamp: Date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[historyDmaHourObj]): Future[historyDmaHourObj] = {
    // create path and map variables
    val path = (addFmt("/dmas/{dmaId}/hours/{timestamp}")
      replaceAll("\\{" + "dmaId" + "\\}", dmaId.toString)
      replaceAll("\\{" + "timestamp" + "\\}", timestamp.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling DmasApi->dmasDmaIdHoursTimestampGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def dmasDmaIdHoursTimestampStarttimestampEndGet(dmaId: Integer,
    timestampStart: Date,
    timestampEnd: Date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[historyDmaHourObj]): Future[historyDmaHourObj] = {
    // create path and map variables
    val path = (addFmt("/dmas/{dmaId}/hours/{timestampStart},{timestampEnd}")
      replaceAll("\\{" + "dmaId" + "\\}", dmaId.toString)
      replaceAll("\\{" + "timestampStart" + "\\}", timestampStart.toString)
      replaceAll("\\{" + "timestampEnd" + "\\}", timestampEnd.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling DmasApi->dmasDmaIdHoursTimestampStarttimestampEndGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }


}
